<?php

class Cheque extends PaymentModule
{
    private $_html = '';
    private $_postErrors = array();

    public  $chequeName;
    public  $address;

    function __construct()
    {
        $this->name = 'cheque';
        $this->tab = 'Payment';
        $this->version = 1.1;

        $config = Configuration::getMultiple(array('CHEQUE_NAME', 'CHEQUE_ADDRESS'));
		if (isset($config['CHEQUE_NAME']))
			$this->chequeName = $config['CHEQUE_NAME'];
		if (isset($config['CHEQUE_ADDRESS']))
			$this->address = $config['CHEQUE_ADDRESS'];

        parent::__construct();

        /* The parent construct is required for translations */
		$this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('Cheque');
        $this->description = $this->l('Module accepting payments by cheque');
		if (!isset($this->address) OR !isset($this->address))
			$this->warning = $this->l('Name and address must be configured in order to use this module correctly');
    }

    function install()
    {
        parent::install();
        $this->registerHook('payment');
        $this->registerHook('paymentReturn');
    }

    function uninstall()
    {
		Configuration::deleteByName('CHEQUE_NAME');
		Configuration::deleteByName('CHEQUE_ADDRESS');
        parent::uninstall();
    }

    private function _postValidation()
    {
        if (empty($_POST['name']))
            $this->_postErrors[] = $this->l('name is required.');
        elseif (empty($_POST['address']))
            $this->_postErrors[] = $this->l('address is required.');
    }

    private function _postProcess()
    {
		Configuration::updateValue('CHEQUE_NAME', $_POST['name']);
		Configuration::updateValue('CHEQUE_ADDRESS', $_POST['address']);

        $this->_html .= '<div class="conf confirm"><img src="../img/admin/ok.gif" alt="'.$this->l('ok').'" /> '.$this->l('Settings updated').'</div>';
    }

    private function _displayCheque()
    {
        $this->_html .= '
        <p><b>'.$this->l('This module allows you to accept payments by cheques.').'</b><br /><br />
        '.$this->l('Confirm manually your order as soon as you receive this cheque.').'<br />'.
        $this->l('If the client chooses this payment mode, the order will change its status into a "Waiting for payment" status.').'</p><br />';
    }

    private function _displayForm()
    {
		if (!isset($_POST['btnSubmit']))
        {
			$_POST['name'] = $this->chequeName;
			$_POST['address'] = $this->address;
        }
			
        $this->_html .=
        '<fieldset><form action="'.$_SERVER['REQUEST_URI'].'" method="post">
            <table border="0" width="500" cellpadding="0" cellspacing="0" id="form">
          		<tr><td width="130" style="height: 35px;">'.$this->l('Name').'</td><td><input type="text" name="name" value="'.(isset($_POST['name']) ? $_POST['name'] : '').'" style="width: 300px;" /></td></tr>
          		<tr><td width="130" style="vertical-align: top;">'.$this->l('Address').'</td>
                  <td><textarea name="address" rows="3" cols="53">'.(isset($_POST['address']) ? $_POST['address'] : $this->l('Please contact us to obtain it.')).'</textarea></td></tr>
                <tr><td colspan="2" align="center"><br /><input class="button" name="btnSubmit" value="'.$this->l('Update settings').'" type="submit" /></td></tr>
            </table>
        </form></fieldset>';
    }

    function getContent()
    {
        $this->_html = '<h2>'.$this->displayName.'</h2>';

        if (!empty($_POST))
        {
            $this->_postValidation();
            if (!count($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors AS $err)
                    $this->_html .= '<div class="alert error">'. $err .'</div>';
        }
        else
            $this->_html .= '<br />';

        $this->_displayCheque();
        $this->_displayForm();

        return $this->_html;
    }

	function execPayment($cart)
	{
		global $cookie, $smarty;

		$smarty->assign(array(
			'currency' => new Currency(Configuration::get('PS_CURRENCY_DEFAULT')),
			'currencies' => Currency::getCurrencies(),
			'total' => number_format($cart->getOrderTotal(true, 3), 2, '.', ''),
			'isoCode' => Language::getIsoById(intval($cookie->id_lang)),
			'chequeName' => $this->chequeName,
			'chequeAddress' => nl2br2($this->address)
		));

		return $this->display(__FILE__, 'payment_execution.tpl');
	}

    function hookPayment($params)
    {
		return $this->display(__FILE__, 'payment.tpl');
    }

    function hookPaymentReturn($params)
    {
		global $smarty;
		$state = $params['objOrder']->getCurrentState();

		if ($state == _PS_OS_CHEQUE_ OR $state == _PS_OS_OUTOFSTOCK_)
			$smarty->assign(array(
				'total_to_pay' => $params['total_to_pay'],
				'currency' => $params['currency'],
				'chequeName' => $this->chequeName,
				'chequeAddress' => nl2br2($this->address),
				'status' => 'ok'
			));
		else
			$smarty->assign('status', 'failed');
        return $this->display(__FILE__, 'payment_return.tpl');
    }
}

?>
